#!/bin/sh
#set -evx
#shopt -s extglob
#
pkgName=amdgpu-userspace
tarZ="tar.gz"
#
errorMsg='echo -e \e[31mERROR::\e[0m'
function okMsg() {
    local msg="$@"
    echo -e "\e[32m $@ \e[0m"
}
if [ -n $1 ]; then
    if [[ "$1" == "-h" || "$1" == "--help" ]]; then printf '%s\n' \
" Script to generate ${pkgName} tarball. Requires wget/rpm2cpio/cpio/bm(optional).
   -h:--help - Prints out this message
   --with-bm - Uses bm packaging tool to create the rpm package(x86_64 only).
               Assumes that ${pkgName}.${tarZ} and ${pkgName}.spec files exist and
               are located in the \$PWD for a successful build.
    -c; --clean - Removes tarball and working directory(amdocl).
    "; exit 0
    elif [[ "$1" == "--with-bm" ]]; then bmON="yes";
#       Checking requirements
        if [ ! -e "${pkgName}.spec" ] || [ ! -e "${pkgName}.${tarZ}" ] ; then bmON="no";
            printf  '%s\n' "Generating rpm files not possible!!! \
                            Missing files:"
            if [ ! -e "${pkgName}.spec" ]; then printf '%s\n' "   ${pkgName}.spec"; fi
            if [ ! -e "${pkgName}.${tarZ}" ]; then printf '%s\n' "   ${pkgName}.${tarZ}"; fi
            exit 1
        fi
        which "bm" > /dev/null 2>&1;
        if [ $? -gt 0 ]; then echo "\"bm\" not found!!! Install \"bm\"."; exit 1; fi
#       Generate rpms with bm
        mkdir -p {SOURCES,SPECS}
        cp ${pkgName}.${tarZ} gettarball2-${pkgName}.sh ./SOURCES
        cp ${pkgName}.spec SPECS
        bm -l SPECS/${pkgName}.spec
        rm -rf BUILD BUILDROOT SOURCES SPECS tmp
        exit 0
    elif [[ "$1" == "-c" || "$1" == "--clean" ]]; then
        if [ -e $(pwd)/amdocl ]; then rm -rf $(pwd)/amdocl; fi
        if [ -e $(pwd)/${pkgName}.${tarZ} ]; then rm -rf $(pwd)/${pkgName}.${tarZ}; fi
        okMsg "Directory cleared!"
    fi
fi
tools=("cpio" "rpm2cpio" "wget")
for t in ${tools[@]}; do tool=${tools[$t]}; which "$tools" > /dev/null 2>&1;
    if [ $? -gt 0 ]; then echo "No $tool found. Install $tool"; exit 1; fi
done
amdgpuVer="21.40.2"
rocmVer="4.5.2"
gpuRel=1350682.el8.x86_64.rpm
rocmRel=40502-164.el8.x86_64.rpm
pwdDir=$(pwd)
if [ ! -e "$pkgName.$tarZ" ]; then
    rpmURL=("https://repo.radeon.com/amdgpu/${amdgpuVer}/rhel/8.5/main/x86_64/libdrm-amdgpu-2.4.107.40502-${gpuRel}" \
        "https://repo.radeon.com/amdgpu/${amdgpuVer}/rhel/8.5/main/x86_64/vulkan-amdgpu-${amdgpuVer}-${gpuRel}" \
        "https://repo.radeon.com/amdgpu/${amdgpuVer}/rhel/8.5/proprietary/x86_64/opencl-legacy-amdgpu-pro-icd-${amdgpuVer}-${gpuRel}" \
        "https://repo.radeon.com/amdgpu/${amdgpuVer}/rhel/8.5/proprietary/x86_64/vulkan-amdgpu-pro-${amdgpuVer}-${gpuRel}" \
        "https://repo.radeon.com/rocm/centos8/${rocmVer}/comgr-2.1.0.${rocmRel}" \
        "https://repo.radeon.com/rocm/centos8/${rocmVer}/hip-runtime-amd-4.4.21432.${rocmRel}" \
        "https://repo.radeon.com/rocm/centos8/${rocmVer}/hsa-rocr-1.4.0.${rocmRel}" \
        "https://repo.radeon.com/rocm/centos8/${rocmVer}/rocm-opencl-2.0.0.${rocmRel}")
    saveDir=$(pwd)/amdocl; mkdir -p "$saveDir"
    pushd "$saveDir"
    if [ ! -d amdDir ]; then mkdir -p amdDir;
        echo "   List of AMD OpenCL sources: " > $pwdDir/ListSources.txt
        pushd amdDir; for pkg in ${rpmURL[@]}; do wget "$pkg"; echo "$pkg" >> $pwdDir/ListSources.txt; done; popd
    fi
    echo "Source packages downloaded in: ${saveDir}/amdDir"
    if [ -d "$pkgName" ]; then rm -rf "$pkgName"; fi
    mkdir -p "$pkgName"
    pushd "$pkgName"
    for pkg in "../amdDir/"*; do rpm2cpio "$pkg" | cpio -dium; done
    echo "Sources unpacked!"
    mkdir -p srcDir; srcDir=srcDir
    mkdir -p ${srcDir}{/usr/lib64,/etc/OpenCL/vendors}
    mkdir -p ${srcDir}/opt/amdgpu{,-pro}/lib64
    mkdir -p ${srcDir}/opt/amdgpu{,-pro}/etc/vulkan/icd.d
#
    mv "opt/rocm-${rocmVer}/opencl/lib/libamdocl64.so" "${srcDir}/usr/lib64"
    mv "opt/rocm-${rocmVer}/hip/lib/libamdhip64.so"* "${srcDir}/usr/lib64"
    mv "opt/rocm-${rocmVer}/lib/libhsa-runtime64.so"* "${srcDir}/usr/lib64"
    mv "opt/rocm-${rocmVer}/lib64/libamd_comgr.so"* "${srcDir}/usr/lib64"
#
    mv "opt/amdgpu/lib64/libdrm_amdgpu.so.1.0.0" "${srcDir}/opt/amdgpu/lib64/libdrm_amdgpu.so.1.0.0"
    pushd ${srcDir}/opt/amdgpu/lib64/; ln -sf libdrm_amdgpu.so.1.0.0 libdrm_amdgpu.so.1; popd
#   OpenCL legacy
    mv "opt/amdgpu-pro/lib64/libamdocl-orca64.so" "${srcDir}/usr/lib64"
    echo libamdocl64.so > ${srcDir}/etc/OpenCL/vendors/amdocl64.icd
    echo libamdocl-orca64.so > ${srcDir}/etc/OpenCL/vendors/amdocl-orca64.icd
#   Vulkan opensrc
    mv "opt/amdgpu/etc/vulkan/icd.d/amd_icd64.json" ${srcDir}/opt/amdgpu/etc/vulkan/icd.d/amd_icd64.json
    mv "opt/amdgpu/lib64/amdvlk64.so" "${srcDir}/opt/amdgpu/lib64"
    pushd "${srcDir}/opt/amdgpu/lib64"; ln -sf amdvlk64.so amdvlk64.so.1.0 ; popd
#   Vulkan pro
    mv "opt/amdgpu-pro/etc/vulkan/icd.d/amd_icd64.json" ${srcDir}/opt/amdgpu-pro/etc/vulkan/icd.d/amd_icd64.json
    mv "opt/amdgpu-pro/lib64/amdvlk64.so.1.0" "${srcDir}/opt/amdgpu-pro/lib64"
    pushd "${srcDir}/opt/amdgpu-pro/lib64"; ln -sf amdvlk64.so.1.0 amdvlk64.so; popd

    okMsg "Files ready to be archived!"
    pushd $srcDir
    if [[ "$tarZ" == "tar.gz" ]]; then
        tar -zcvf "${saveDir}"/../"${pkgName}.${tarZ}" *
    else
        tar -cvf "${saveDir}"/../"${pkgName}.tar" *
        xz -v -T0 -9e -f "${saveDir}"/../"${pkgName}.tar"
    fi
    popd
    popd
    okMsg "Files archived in ${pkgName}.${tarZ} !"
    exit 0
fi


